@@ -4,17 +4,16 @@
 //!
 //! [`IndexMap`]: https://docs.rs/indexmap/latest/indexmap/map/struct.IndexMap.html
 
-//use serde::{de, ser};
 use indexmap::{map, IndexMap};
 use std::{
     fmt::{self, Debug},
-    iter::FromIterator,
     ops,
 };
 
 use crate::Value;
 
 /// Represents a plist dictionary type.
+#[derive(Clone, Default, PartialEq)]
 pub struct Dictionary {
     map: IndexMap<String, Value>,
 }
@@ -67,7 +66,28 @@ impl Dictionary {
     /// in the dictionary.
     #[inline]
     pub fn remove(&mut self, key: &str) -> Option<Value> {
-        self.map.remove(key)
+        self.map.swap_remove(key)
+    }
+
+    /// Scan through each key-value pair in the map and keep those where the
+    /// closure `keep` returns `true`.
+    #[inline]
+    pub fn retain<F>(&mut self, keep: F)
+    where
+        F: FnMut(&String, &mut Value) -> bool,
+    {
+        self.map.retain(keep)
+    }
+
+    /// Sort the dictionary keys.
+    ///
+    /// This uses the default ordering defined on [`str`].
+    ///
+    /// This function is useful if you are serializing to XML, and wish to
+    /// ensure a consistent key order.
+    #[inline]
+    pub fn sort_keys(&mut self) {
+        self.map.sort_keys()
     }
 
     /// Gets the given key's corresponding entry in the dictionary for in-place manipulation.
@@ -140,31 +160,6 @@ impl Dictionary {
     }
 }
 
-impl Default for Dictionary {
-    #[inline]
-    fn default() -> Self {
-        Dictionary {
-            map: Default::default(),
-        }
-    }
-}
-
-impl Clone for Dictionary {
-    #[inline]
-    fn clone(&self) -> Self {
-        Dictionary {
-            map: self.map.clone(),
-        }
-    }
-}
-
-impl PartialEq for Dictionary {
-    #[inline]
-    fn eq(&self, other: &Self) -> bool {
-        self.map.eq(&other.map)
-    }
-}
-
 /// Access an element of this dictionary. Panics if the given key is not present in the dictionary.
 ///
 /// ```
@@ -210,71 +205,13 @@ impl Debug for Dictionary {
     }
 }
 
-/*impl ser::Serialize for Dictionary {
-    #[inline]
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: ser::Serializer,
-    {
-        use serde::ser::SerializeMap;
-        let mut map = try!(serializer.serialize_map(Some(self.len())));
-        for (k, v) in self {
-            try!(map.serialize_key(k));
-            try!(map.serialize_value(v));
-        }
-        map.end()
-    }
-}
-
-impl<'de> de::Deserialize<'de> for Dictionary {
-    #[inline]
-    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: de::Deserializer<'de>,
-    {
-        struct Visitor;
-
-        impl<'de> de::Visitor<'de> for Visitor {
-            type Value = Dictionary;
-
-            fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
-                formatter.write_str("a map")
-            }
-
-            #[inline]
-            fn visit_unit<E>(self) -> Result<Self::Value, E>
-            where
-                E: de::Error,
-            {
-                Ok(Dictionary::new())
-            }
-
-            #[inline]
-            fn visit_map<V>(self, mut visitor: V) -> Result<Self::Value, V::Error>
-            where
-                V: de::MapAccess<'de>,
-            {
-                let mut values = Dictionary::new();
-
-                while let Some((key, value)) = try!(visitor.next_entry()) {
-                    values.insert(key, value);
-                }
-
-                Ok(values)
-            }
-        }
-
-        deserializer.deserialize_map(Visitor)
-    }
-}*/
-
-impl FromIterator<(String, Value)> for Dictionary {
-    fn from_iter<T>(iter: T) -> Self
-    where
-        T: IntoIterator<Item = (String, Value)>,
-    {
+impl<K: Into<String>, V: Into<Value>> FromIterator<(K, V)> for Dictionary {
+    fn from_iter<I: IntoIterator<Item = (K, V)>>(iter: I) -> Self {
         Dictionary {
-            map: FromIterator::from_iter(iter),
+            map: iter
+                .into_iter()
+                .map(|(k, v)| (k.into(), v.into()))
+                .collect(),
         }
     }
 }
@@ -602,7 +539,7 @@ impl<'a> OccupiedEntry<'a> {
     /// ```
     #[inline]
     pub fn remove(self) -> Value {
-        self.occupied.remove()
+        self.occupied.swap_remove()
     }
 }
 
@@ -694,3 +631,103 @@ pub struct ValuesMut<'a> {
 }
 
 delegate_iterator!((ValuesMut<'a>) => &'a mut Value);
+
+#[cfg(feature = "serde")]
+pub mod serde_impls {
+    use serde::{de, ser};
+    use std::fmt;
+
+    use crate::Dictionary;
+
+    impl ser::Serialize for Dictionary {
+        #[inline]
+        fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+        where
+            S: ser::Serializer,
+        {
+            use serde::ser::SerializeMap;
+            let mut map = serializer.serialize_map(Some(self.len()))?;
+            for (k, v) in self {
+                map.serialize_key(k)?;
+                map.serialize_value(v)?;
+            }
+            map.end()
+        }
+    }
+
+    impl<'de> de::Deserialize<'de> for Dictionary {
+        #[inline]
+        fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+        where
+            D: de::Deserializer<'de>,
+        {
+            struct Visitor;
+
+            impl<'de> de::Visitor<'de> for Visitor {
+                type Value = Dictionary;
+
+                fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+                    formatter.write_str("a map")
+                }
+
+                #[inline]
+                fn visit_unit<E>(self) -> Result<Self::Value, E>
+                where
+                    E: de::Error,
+                {
+                    Ok(Dictionary::new())
+                }
+
+                #[inline]
+                fn visit_map<V>(self, mut visitor: V) -> Result<Self::Value, V::Error>
+                where
+                    V: de::MapAccess<'de>,
+                {
+                    let mut values = Dictionary::new();
+
+                    while let Some((key, value)) = visitor.next_entry()? {
+                        values.insert(key, value);
+                    }
+
+                    Ok(values)
+                }
+            }
+
+            deserializer.deserialize_map(Visitor)
+        }
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use super::Dictionary;
+
+    #[test]
+    fn from_hash_map_to_dict() {
+        let dict: Dictionary = [
+            ("Doge", "Shiba Inu"),
+            ("Cheems", "Shiba Inu"),
+            ("Walter", "Bull Terrier"),
+            ("Perro", "Golden Retriever"),
+        ]
+        .into_iter()
+        .collect();
+
+        assert_eq!(
+            dict.get("Doge").and_then(|v| v.as_string()),
+            Some("Shiba Inu")
+        );
+        assert_eq!(
+            dict.get("Cheems").and_then(|v| v.as_string()),
+            Some("Shiba Inu")
+        );
+        assert_eq!(
+            dict.get("Walter").and_then(|v| v.as_string()),
+            Some("Bull Terrier")
+        );
+        assert_eq!(
+            dict.get("Perro").and_then(|v| v.as_string()),
+            Some("Golden Retriever")
+        );
+    }
+}
